@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
- * Copyright (c) 2002-2005,2009 Mellanox Technologies LTD. All rights reserved.
+ * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -126,6 +126,7 @@ typedef struct osm_sm {
 	cl_dispatcher_t *p_disp;
 	cl_plock_t *p_lock;
 	atomic32_t sm_trans_id;
+	uint16_t mlids_init_max;
 	unsigned mlids_req_max;
 	uint8_t *mlids_req;
 	osm_sm_mad_ctrl_t mad_ctrl;
@@ -134,6 +135,7 @@ typedef struct osm_sm {
 	cl_disp_reg_handle_t sweep_fail_disp_h;
 	cl_disp_reg_handle_t ni_disp_h;
 	cl_disp_reg_handle_t pi_disp_h;
+	cl_disp_reg_handle_t gi_disp_h;
 	cl_disp_reg_handle_t nd_disp_h;
 	cl_disp_reg_handle_t si_disp_h;
 	cl_disp_reg_handle_t lft_disp_h;
@@ -143,6 +145,7 @@ typedef struct osm_sm {
 	cl_disp_reg_handle_t slvl_disp_h;
 	cl_disp_reg_handle_t vla_disp_h;
 	cl_disp_reg_handle_t pkey_disp_h;
+	cl_disp_reg_handle_t mlnx_epi_disp_h;
 } osm_sm_t;
 /*
 * FIELDS
@@ -399,6 +402,7 @@ ib_api_status_t osm_sm_bind(IN osm_sm_t * p_sm, IN ib_net64_t port_guid);
 */
 ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 			    IN ib_net16_t attr_id, IN ib_net32_t attr_mod,
+			    IN boolean_t find_mkey, ib_net64_t m_key,
 			    IN cl_disp_msgid_t err_msg,
 			    IN const osm_madw_context_t * p_context);
 /*
@@ -416,6 +420,13 @@ ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 *	attr_mod
 *		[in] Attribute modifier for this request.
 *
+*	find_mkey
+*		[in] Flag to indicate whether the M_Key should be looked up for
+*		     this MAD.
+* 	m_key
+* 		[in] M_Key value to be send with this MAD. Applied, only when
+* 		     find_mkey is FALSE.
+*
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
@@ -431,6 +442,81 @@ ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 *	The response from the node will be routed through the Dispatcher
 *	to the appropriate receive controller object.
 *********/
+
+/****f* OpenSM: SM/osm_send_req_mad
+* NAME
+*       osm_send_req_mad
+*
+* DESCRIPTION
+*	Starts the process to transmit a preallocated/predefined directed route
+*	Set() request.
+*
+* SYNOPSIS
+*/
+void osm_send_req_mad(IN osm_sm_t * sm, IN osm_madw_t *p_madw);
+/*
+* PARAMETERS
+*	sm
+*		[in] Pointer to an osm_sm_t object.
+*	p_madw
+*		[in] Pointer to a preallocated MAD buffer
+*
+*********/
+
+/***f* OpenSM: SM/osm_prepare_req_set
+* NAME
+*	osm_prepare_req_set
+*
+* DESCRIPTION
+*	Preallocate and fill a directed route Set() MAD w/o sending it.
+*
+* SYNOPSIS
+*/
+osm_madw_t *osm_prepare_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
+				IN const uint8_t * p_payload,
+				IN size_t payload_size, IN ib_net16_t attr_id,
+				IN ib_net32_t attr_mod, IN boolean_t find_mkey,
+				IN ib_net64_t m_key, IN cl_disp_msgid_t err_msg,
+				IN const osm_madw_context_t * p_context);
+/*
+* PARAMETERS
+*	sm
+*		[in] Pointer to an osm_sm_t object.
+*
+*	p_path
+*		[in] Pointer to the directed route path of the recipient.
+*
+*	p_payload
+*		[in] Pointer to the SMP payload to send.
+*
+*	payload_size
+*		[in] The size of the payload to be copied to the SMP data field.
+*
+*	attr_id
+*		[in] Attribute ID to request.
+*
+*	attr_mod
+*		[in] Attribute modifier for this request.
+*
+*	find_mkey
+*		[in] Flag to indicate whether the M_Key should be looked up for
+*		     this MAD.
+* 	m_key
+* 		[in] M_Key value to be send with this MAD. Applied, only when
+* 		     find_mkey is FALSE.
+*
+*	err_msg
+*		[in] Message id with which to post this MAD if an error occurs.
+*
+*	p_context
+*		[in] Mad wrapper context structure to be copied into the wrapper
+*		     context, and thus visible to the recipient of the response.
+*
+* RETURN VALUES
+*	Pointer the MAD buffer in case of success and NULL in case of failure.
+*
+*********/
+
 /****f* OpenSM: SM/osm_req_set
 * NAME
 *	osm_req_set
@@ -443,7 +529,8 @@ ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 ib_api_status_t osm_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 			    IN const uint8_t * p_payload,
 			    IN size_t payload_size, IN ib_net16_t attr_id,
-			    IN ib_net32_t attr_mod, IN cl_disp_msgid_t err_msg,
+			    IN ib_net32_t attr_mod, IN boolean_t find_mkey,
+			    IN ib_net64_t m_key, IN cl_disp_msgid_t err_msg,
 			    IN const osm_madw_context_t * p_context);
 /*
 * PARAMETERS
@@ -465,6 +552,14 @@ ib_api_status_t osm_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 *	attr_mod
 *		[in] Attribute modifier for this request.
 *
+*	find_mkey
+*		[in] Flag to indicate whether the M_Key should be looked up for
+*		     this MAD.
+*
+* 	m_key
+* 		[in] M_Key value to be send with this MAD. Applied, only when
+* 		     find_mkey is FALSE.
+*
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
