@@ -5,6 +5,7 @@
  * Copyright (c) 2008 Xsigo Systems Inc.  All rights reserved.
  * Copyright (c) 2009 Sun Microsystems, Inc. All rights reserved.
  * Copyright (c) 2010 HNR Consulting. All rights reserved.
+ * Copyright (C) 2012-2013 Tokyo Institute of Technology. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -59,12 +60,7 @@
 #include <opensm/osm_switch.h>
 #include <opensm/osm_helper.h>
 #include <opensm/osm_msgdef.h>
-
-typedef struct osm_mcast_work_obj {
-	cl_list_item_t list_item;
-	osm_port_t *p_port;
-	cl_map_item_t map_item;
-} osm_mcast_work_obj_t;
+#include <opensm/osm_mcast_mgr.h>
 
 static osm_mcast_work_obj_t *mcast_work_obj_new(IN osm_port_t * p_port)
 {
@@ -89,16 +85,16 @@ static void mcast_work_obj_delete(IN osm_mcast_work_obj_t * p_wobj)
 	free(p_wobj);
 }
 
-static int make_port_list(cl_qlist_t * list, osm_mgrp_box_t * mbox)
+int osm_mcast_make_port_list_and_map(cl_qlist_t * list, cl_qmap_t * map,
+				     osm_mgrp_box_t * mbox)
 {
-	cl_qmap_t map;
 	cl_map_item_t *map_item;
 	cl_list_item_t *list_item;
 	osm_mgrp_t *mgrp;
 	osm_mcm_port_t *mcm_port;
 	osm_mcast_work_obj_t *wobj;
 
-	cl_qmap_init(&map);
+	cl_qmap_init(map);
 	cl_qlist_init(list);
 
 	for (list_item = cl_qlist_head(&mbox->mgrp_list);
@@ -111,50 +107,33 @@ static int make_port_list(cl_qlist_t * list, osm_mgrp_box_t * mbox)
 			/* Acquire the port object for this port guid, then
 			   create the new worker object to build the list. */
 			mcm_port = cl_item_obj(map_item, mcm_port, map_item);
-			if (cl_qmap_get(&map, mcm_port->port->guid) !=
-			    cl_qmap_end(&map))
+			if (cl_qmap_get(map, mcm_port->port->guid) !=
+			    cl_qmap_end(map))
 				continue;
 			wobj = mcast_work_obj_new(mcm_port->port);
 			if (!wobj)
 				return -1;
 			cl_qlist_insert_tail(list, &wobj->list_item);
-			cl_qmap_insert(&map, mcm_port->port->guid,
+			cl_qmap_insert(map, mcm_port->port->guid,
 				       &wobj->map_item);
 		}
 	}
 	return 0;
 }
 
-static void drop_port_list(cl_qlist_t * list)
+void osm_mcast_drop_port_list(cl_qlist_t * list)
 {
 	while (cl_qlist_count(list))
 		mcast_work_obj_delete((osm_mcast_work_obj_t *)
 				      cl_qlist_remove_head(list));
 }
 
-/**********************************************************************
- Recursively remove nodes from the tree
- *********************************************************************/
-static void mcast_mgr_purge_tree_node(IN osm_mtree_node_t * p_mtn)
-{
-	uint8_t i;
-
-	for (i = 0; i < p_mtn->max_children; i++) {
-		if (p_mtn->child_array[i] &&
-		    (p_mtn->child_array[i] != OSM_MTREE_LEAF))
-			mcast_mgr_purge_tree_node(p_mtn->child_array[i]);
-		p_mtn->child_array[i] = NULL;
-	}
-
-	free(p_mtn);
-}
-
 void osm_purge_mtree(osm_sm_t * sm, IN osm_mgrp_box_t * mbox)
 {
 	OSM_LOG_ENTER(sm->p_log);
 
 	if (mbox->root)
-		mcast_mgr_purge_tree_node(mbox->root);
+		osm_mtree_destroy(mbox->root);
 	mbox->root = NULL;
 
 	OSM_LOG_EXIT(sm->p_log);
@@ -330,7 +309,7 @@ static osm_switch_t *mcast_mgr_find_optimal_switch(osm_sm_t * sm,
 /**********************************************************************
    This function returns the existing or optimal root switch for the tree.
 **********************************************************************/
-static osm_switch_t *mcast_mgr_find_root_switch(osm_sm_t * sm, cl_qlist_t *list)
+osm_switch_t *osm_mcast_mgr_find_root_switch(osm_sm_t * sm, cl_qlist_t *list)
 {
 	osm_switch_t *p_sw = NULL;
 
@@ -352,6 +331,7 @@ static int mcast_mgr_set_mft_block(osm_sm_t * sm, IN osm_switch_t * p_sw,
 				   uint32_t block_num, uint32_t position)
 {
 	osm_node_t *p_node;
+	osm_physp_t *p_physp;
 	osm_dr_path_t *p_path;
 	osm_madw_context_t context;
 	ib_api_status_t status;
@@ -370,7 +350,8 @@ static int mcast_mgr_set_mft_block(osm_sm_t * sm, IN osm_switch_t * p_sw,
 
 	CL_ASSERT(p_node);
 
-	p_path = osm_physp_get_dr_path_ptr(osm_node_get_physp_ptr(p_node, 0));
+	p_physp = osm_node_get_physp_ptr(p_node, 0);
+	p_path = osm_physp_get_dr_path_ptr(p_physp);
 
 	/*
 	   Send multicast forwarding table blocks to the switch
@@ -394,11 +375,13 @@ static int mcast_mgr_set_mft_block(osm_sm_t * sm, IN osm_switch_t * p_sw,
 
 		status = osm_req_set(sm, p_path, (void *)block, sizeof(block),
 				     IB_MAD_ATTR_MCAST_FWD_TBL,
-				     cl_hton32(block_id_ho), CL_DISP_MSGID_NONE,
-				     &context);
+				     cl_hton32(block_id_ho), FALSE,
+				     ib_port_info_get_m_key(&p_physp->port_info),
+				     CL_DISP_MSGID_NONE, &context);
 		if (status != IB_SUCCESS) {
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A02: "
-				"Sending multicast fwd. tbl. block to %s failed (%s)\n",
+				"Sending multicast fwd. tbl. block 0x%X to %s "
+				"failed (%s)\n", block_id_ho,
 				p_node->print_desc, ib_get_err_str(status));
 			ret = -1;
 		}
@@ -459,7 +442,7 @@ static void mcast_mgr_subdivide(osm_sm_t * sm, uint16_t mlid_ho,
 			continue;
 		}
 
-		if (port_num > array_size) {
+		if (port_num >= array_size) {
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A04: "
 				"Error routing MLID 0x%X through switch 0x%"
 				PRIx64 " %s\n"
@@ -470,8 +453,6 @@ static void mcast_mgr_subdivide(osm_sm_t * sm, uint16_t mlid_ho,
 				cl_ntoh16(osm_port_get_base_lid
 					  (p_wobj->p_port)));
 			mcast_work_obj_delete(p_wobj);
-			/* This is means OpenSM has a bug. */
-			CL_ASSERT(FALSE);
 			continue;
 		}
 
@@ -494,7 +475,7 @@ static void mcast_mgr_purge_list(osm_sm_t * sm, cl_qlist_t * list)
 				osm_port_get_guid(wobj->p_port));
 		}
 	}
-	drop_port_list(list);
+	osm_mcast_drop_port_list(list);
 }
 
 /**********************************************************************
@@ -539,7 +520,7 @@ static osm_mtree_node_t *mcast_mgr_branch(osm_sm_t * sm, uint16_t mlid_ho,
 	depth++;
 
 	if (depth >= 64) {
-		OSM_LOG(sm->p_log, OSM_LOG_ERROR,
+		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A21: "
 			"Maximal hops number is reached for MLID 0x%x."
 			" Break processing.", mlid_ho);
 		mcast_mgr_purge_list(sm, p_list);
@@ -725,6 +706,7 @@ static ib_api_status_t mcast_mgr_build_spanning_tree(osm_sm_t * sm,
 						     osm_mgrp_box_t * mbox)
 {
 	cl_qlist_t port_list;
+	cl_qmap_t port_map;
 	uint32_t num_ports;
 	osm_switch_t *p_sw;
 	ib_api_status_t status = IB_SUCCESS;
@@ -741,7 +723,7 @@ static ib_api_status_t mcast_mgr_build_spanning_tree(osm_sm_t * sm,
 	osm_purge_mtree(sm, mbox);
 
 	/* build the first "subset" containing all member ports */
-	if (make_port_list(&port_list, mbox)) {
+	if (osm_mcast_make_port_list_and_map(&port_list, &port_map, mbox)) {
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A10: "
 			"Insufficient memory to make port list\n");
 		status = IB_ERROR;
@@ -753,7 +735,7 @@ static ib_api_status_t mcast_mgr_build_spanning_tree(osm_sm_t * sm,
 		OSM_LOG(sm->p_log, OSM_LOG_VERBOSE,
 			"MLID 0x%X has %u members - nothing to do\n",
 			mbox->mlid, num_ports);
-		drop_port_list(&port_list);
+		osm_mcast_drop_port_list(&port_list);
 		goto Exit;
 	}
 
@@ -773,12 +755,12 @@ static ib_api_status_t mcast_mgr_build_spanning_tree(osm_sm_t * sm,
 	   Locate the switch around which to create the spanning
 	   tree for this multicast group.
 	 */
-	p_sw = mcast_mgr_find_root_switch(sm, &port_list);
+	p_sw = osm_mcast_mgr_find_root_switch(sm, &port_list);
 	if (p_sw == NULL) {
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A08: "
 			"Unable to locate a suitable switch for group 0x%X\n",
 			mbox->mlid);
-		drop_port_list(&port_list);
+		osm_mcast_drop_port_list(&port_list);
 		status = IB_ERROR;
 		goto Exit;
 	}
@@ -1070,10 +1052,13 @@ static void mcast_mgr_set_mfttop(IN osm_sm_t * sm, IN osm_switch_t * p_sw)
 		context.si_context.light_sweep = FALSE;
 		context.si_context.node_guid = osm_node_get_node_guid(p_node);
 		context.si_context.set_method = TRUE;
+		context.si_context.lft_top_change = FALSE;
 
 		status = osm_req_set(sm, p_path, (uint8_t *) & si,
 				     sizeof(si), IB_MAD_ATTR_SWITCH_INFO,
-				     0, CL_DISP_MSGID_NONE, &context);
+				     0, FALSE,
+				     ib_port_info_get_m_key(&p_physp->port_info),
+				     CL_DISP_MSGID_NONE, &context);
 
 		if (status != IB_SUCCESS)
 			OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 0A1B: "
